package middleware

import "github.com/ThreeDotsLabs/watermill/message"

type IgnoreErrors struct {
	ignoredErrors map[string]struct{}
}

func NewIgnoreErrors(errs []error) IgnoreErrors {
	errsMap := make(map[string]struct{}, len(errs))

	for _, err := range errs {
		errsMap[err.Error()] = struct{}{}
	}

	return IgnoreErrors{errsMap}
}

func (i IgnoreErrors) Middleware(h message.HandlerFunc) message.HandlerFunc {
	return func(msg *message.Message) ([]*message.Message, error) {
		events, err := h(msg)
		if err != nil {
			if _, ok := i.ignoredErrors[err.Error()]; ok {
				return events, nil
			}

			return events, err
		}

		return events, nil
	}
}
